@@ -3,9 +3,9 @@
 # (keep hashbang line for `make install`)
 
 #
-# git timestamp — Independent GIT Timestamping client
+# git timestamp — Zeitgitter GIT Timestamping client
 #
-# Copyright (C) 2019, 2020 Marcel Waldvogel
+# Copyright (C) 2019-2021 Marcel Waldvogel
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published
@@ -23,7 +23,7 @@
 
 # This has not been modularized for ease of installation
 
-import argparse
+import configargparse
 import distutils.util
 import os
 import re
@@ -36,10 +36,10 @@ import gnupg  # Provided e.g. by `pip install python-gnupg` (try with `pip2`/`pi
 import pygit2 as git
 import requests
 
-VERSION = '1.0.4'
+VERSION = '1.0.6'
 
 
-class GitArgumentParser(argparse.ArgumentParser):
+class GitArgumentParser(configargparse.ArgumentParser):
     """Insert git config options between command line and default.
 
     WARNING: There is no way to handle custom actions correctly by default, so
@@ -62,7 +62,7 @@ class GitArgumentParser(argparse.ArgumentParser):
             gitopt = kwargs['gitopt']
             try:
                 if 'action' in kwargs and issubclass(kwargs['action'],
-                                                     argparse.Action):
+                                                     configargparse.Action):
                     try:
                         val = kwargs['action'].convert_default(repo.config[gitopt])
                     except AttributeError:
@@ -107,7 +107,7 @@ def timestamp_branch_name(fields):
     return 'zeitgitter-timestamps'
 
 
-class DefaultTrueIfPresent(argparse.Action):
+class DefaultTrueIfPresent(configargparse.Action):
     def __call__(self, parser, namespace, values, option_string=None):
         if values is None:
             values = True
@@ -115,7 +115,7 @@ class DefaultTrueIfPresent(argparse.Action):
             try:
                 values = self.convert_default(values)
             except ValueError:
-                raise argparse.ArgumentError(self, "Requires boolean value")
+                raise configargparse.ArgumentError(self, "Requires boolean value")
         setattr(namespace, self.dest, values)
 
     @classmethod
@@ -136,6 +136,7 @@ def expanded_aliases():
 def get_args():
     """Parse command line and git config parameters"""
     parser = GitArgumentParser(
+        auto_env_var_prefix='timestamp_',
         add_help=False,
         description="""Interface to Zeitgitter, the network of
                     independent GIT timestampers.""",
@@ -144,7 +145,10 @@ def get_args():
             and `--branch` for frequent timestamping.
             `bool` values can be specified as true/false/yes/no/0/1.
             Arguments with optional `bool` options default to true if
-            the argument is present, false if absent.""")
+            the argument is present, false if absent.
+            Environment variable `ZEITGITTER_FAKE_TIME` can be used for
+            repeatable tests against a local Zeitgitter server under test,
+            see there.""")
     parser.add('--help', '-h',
                action='help',
                help="""Show this help message and exit. When called as
@@ -209,15 +213,6 @@ def get_args():
         sys.exit("Timestamping explicitely disabled")
     if arg.require_enable and arg.enable != True:
         sys.exit("Timestamping not explicitely enabled")
-    if arg.server in server_aliases:
-        arg.server = server_aliases[arg.server]
-    if ':' not in arg.server:
-        arg.server = 'https://' + arg.server
-    if arg.tag is None and arg.branch is None:
-        # Automatically derive branch name
-        # Split URL on '.' or '/'
-        fields = arg.server.replace('/', '.').split('.')
-        arg.branch = timestamp_branch_name(fields[1:])
     return arg
 
 
@@ -609,11 +604,37 @@ def main():
                  "    Possible remedy: `pip uninstall gnupg;"
                  " pip install python-gnupg`\n"
                  "    (try `pip2`/`pip3` if it does not work with `pip`)")
-    (keyid, name) = get_keyid(args)
-    if args.tag:
-        timestamp_tag(repo, keyid, name, args)
+    if args.tag is not None or args.branch is not None:
+        # Single tag or branch against one timestamping server
+        if ',' in args.server:
+            (server, rest) = args.server.split(',', 1)
+            args.server = server
+            print(f"WARNING: Cannot timestamp single tag/branch against"
+                    " multiple servers;\nonly timestamping against {server}")
+        (keyid, name) = get_keyid(args)
+        if args.tag:
+            timestamp_tag(repo, keyid, name, args)
+        else:
+            timestamp_branch(repo, keyid, name, args)
     else:
-        timestamp_branch(repo, keyid, name, args)
+        # Automatic branch, with support for multiple timestamping servers
+        success = True
+        for server in args.server.split(','):
+            if server in server_aliases:
+                server = server_aliases[server]
+            if ':' not in server:
+                server = 'https://' + server
+            fields = server.replace('/', '.').split('.')
+            args.branch = timestamp_branch_name(fields[1:])
+            args.server = server
+            try:
+                (keyid, name) = get_keyid(args)
+                timestamp_branch(repo, keyid, name, args)
+            except SystemExit as e:
+                sys.stderr.write(e.code + '\n')
+                success = False
+        if not success:
+            sys.exit(1)
 
 
 if __name__ == "__main__":
